#!/usr/bin/perl

# sms.pl 1.0 Copyright (c) Valery Dachev
#
# mailto:valery@zonebg.com
# http://valery.zonebg.com
#
# Distributed under the terms of the GPL license
# as published by the Free Software Foundation.

use Net::SMTP;

@confFiles	= (
	"$ENV{HOME}/.sms.conf",
	"$ENV{HOME}/.smsrc",
	"/etc/sms.conf",
	"/etc/smsrc",
);

%globals		= (
	verbose	=> 0,
	timeout	=> 15,
	server	=> localhost
);
%servers	= ();
%addresses	= ();

sub printUsage
{
	print "sms.pl 1.0 Copyright (c) 2002 Valery Dachev\n";
	print "\n";
	print "Usage: sms.pl <number/contact> <message>\n";
	print "\n";
}

sub readCmdLine
{
	if( $ARGV[0] =~ /^[\s\t]*$/ )
	{
		print "ERROR: No number or contact entered !\n";
		&printUsage();
		exit 1;
	}
	else
	{
		$contact = $ARGV[0];
	}
	
	$message = $ARGV[1];
	for( my $i = 2; $i <= $#ARGV; $i++ )
	{
		$message = "$message $ARGV[$i]"
	}
	if( $message =~ /^[\s\t]*$/ )
	{
		print "ERROR: No message entered !\n";
		&printUsage();
		exit 1;
	}
}

sub readConfig
{
	my $found = 0;
	foreach ( @confFiles )
	{
		if( -f $_ )
		{
			$found = 1;
			$confFile = $_;
		}
	}
	
	if( $found == 0 )
	{
		print "ERROR: Couldn't find a config file to read.\n";
		exit;
	}

	open( HANDLE, "$confFile" );
	
	my $reading = "globals";
	my $i = 0;
	while( ! eof( HANDLE ) )
	{
		$i++;
		$line = <HANDLE>;
		chomp $line;
		
		if( $line =~ /[\t\s]*\[([^\]]+)\][\t\s]*/i )
		{
			$reading = lc $1;
		}
		elsif( $line =~ /^[\t\s]*;.*$/ )
		{
			# comments in config file
		}
		elsif( ! $line =~ /^[\t\s]*$/ )
		{
			if( $reading eq "globals" )
			{
				if( $line =~ /^[\t\s]*([^\t\s=]+)[\t\s]*=[\t\s]*(.+)[\t\s]*$/ )
				{
					$globals{lc $1} = $2;
				}
				else
				{
					print "ERROR on line #$i of $confFile while reading '$reading'.\n";
					exit 1;
				}
			}
			if( $reading eq "servers" )
			{
				if( $line =~ /^[\t\s]*([0-9]+)[\t\s]*=[\t\s]*(mailto:|)([a-zA-Z0-9\-\.?]+@[a-zA-Z0-9\-\.]+)[\t\s]*$/ )
				{
					$servers{$1} = $3;
				}
				else
				{
					print "ERROR on line #$i of $confFile while reading '$reading'.\n";
					exit;
				}	
			}
			if( $reading eq "contacts" )
			{
				if( $line =~ /^[\t\s]*([^\t\s=]+)[\t\s]*=[\t\s]*([0-9]+)[\t\s]*$/ )
				{
					$contacts{lc $1} = $2;
				}
				else
				{
					print "ERROR on line #$i of $confFile while reading '$reading'.\n";
					exit 1;
				}
			}
		}
	}
	
	close( HANDLE );
}

sub contactLookup
{
	my $name = shift;

	$name = lc $name;

	if( exists( $contacts{$name} ) )
	{
		return $contacts{$name};
	}
	else
	{
		return 0;
	}
}

sub serverLookup
{
	my $number = shift;

	foreach my $key ( keys %servers )
	{
		if( index( $number, $key ) == 0 )
		{
			return $key;
		}
	}
	
	return "";
}

sub addressMake
{
	my $number = shift;
	my $server = shift;
	
	my $j = length $number;
	my $i = length $servers{$server};
	for( my $i = $i - 1; $i >= 0; $i-- )
	{
		my $char = substr( $servers{$server}, $i, 1 );
		if( $char eq "?" )
		{
			$j--;
			$repl = substr( $number, $j, 1 );
			substr( $servers{$server}, $i, 1, $repl );
			if( $j < 0 )
			{
				return "";
			}
		}
	}

	return $servers{$server};
}

#### REAL PROGRAM STARTS HERE

&readCmdLine();
&readConfig();

$number = &contactLookup( $contact );
if( $number == 0 )
{
	if( $globals[verbose] > 0 )
		{ print "Couldn't find '$contact' in address book.\n"; }
	
	if( $contact =~ /^[0-9]+$/ )
	{
		$number = $contact;
	}
	else
	{
		print "ERROR: invalid number ($contact).\n";
		exit 2;
	}
}

$server = &serverLookup( $number );
if( $server eq "" )
{
	print "ERROR: Couldn't find appropriate server for '$number'.\n";
	exit 3;
}

$address = &addressMake( $number, $server );
if( $address eq "" )
{
	print "ERROR: Couldn't use the selected server for '$number'.\n";
	exit 4;
}

$MailFrom = $globals{from};
$MailReplyTo = $globals{reply-to};
$MailTo = $address;
$MailContents = $message;

if( $globals{verbose} > 0 )
	{ print( "Connecting server '$globals{server}' with timeout '$globals{timeout}' ...\n" ); }

$smtp = Net::SMTP->new( $globals{server},
	Timeout => $globals{timeout},
	Debug => $globals{verbose} - 1
);

if( $globals{verbose} > 0 )
	{ print( "Setting mail from field to be '$MailFrom' ...\n" ); }

$smtp->mail( $MailFrom );

if( $globals{verbose} > 0 )
	{ print( "Setting mail to field to be '$MailTo' ...\n" ); }

$smtp->to( $MailTo );

if( $globals{verbose} > 0 )
	{ print( "Start sending message ...\n" ); }

$smtp->data();
$smtp->datasend( "From: $MailFrom\n" );
$smtp->datasend( "Reply-To: $MailReplyTo\n" );
$smtp->datasend( "To: $MailTo\n" );
$smtp->datasend( "\n$MailContents\n" );

$return = $smtp->dataend();

if( $globals{verbose} > 0 )
	{ print( "Returned '$return'.\n" ); }
